@@ -1,7 +1,7 @@
 /*
- * $XFree86: $
+ * $RCSId: xc/lib/fontconfig/src/fcdefault.c,v 1.2 2002/07/09 22:08:14 keithp Exp $
  *
- * Copyright  2001 Keith Packard, member of The XFree86 Project, Inc.
+ * Copyright  2001 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -55,6 +55,9 @@ FcDefaultSubstitute (FcPattern *pattern)
 	}
     }
 
+    if (FcPatternGet (pattern, FC_WIDTH, 0, &v) == FcResultNoMatch)
+	FcPatternAddInteger (pattern, FC_WIDTH, FC_WIDTH_NORMAL);
+
     for (i = 0; i < NUM_FC_BOOL_DEFAULTS; i++)
 	if (FcPatternGet (pattern, FcBoolDefaults[i].field, 0, &v) == FcResultNoMatch)
 	    FcPatternAddBool (pattern, FcBoolDefaults[i].field, FcBoolDefaults[i].value);
@@ -137,4 +140,8 @@ FcDefaultSubstitute (FcPattern *pattern)
 		FcPatternAddString (pattern, FC_LANG, (FcChar8 *) lang);
 	}
     }
+    if (FcPatternGet (pattern, FC_FONTVERSION, 0, &v) == FcResultNoMatch)
+    {
+	FcPatternAddInteger (pattern, FC_FONTVERSION, 0x7fffffff);
+    }
 }
